@@ -1,9 +1,9 @@
 /*
- * "$Id: ipp.c,v 1.87 2004/08/26 18:25:50 mike Exp $"
+ * "$Id: ipp.c,v 1.90 2005/01/03 19:29:44 mike Exp $"
  *
  *   IPP backend for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2004 by Easy Software Products, all rights reserved.
+ *   Copyright 1997-2005 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -15,7 +15,7 @@
  *       Attn: CUPS Licensing Information
  *       Easy Software Products
  *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3142 USA
+ *       Hollywood, Maryland 20636 USA
  *
  *       Voice: (301) 373-9600
  *       EMail: cups-info@cups.org
@@ -215,11 +215,12 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
   * Extract the hostname and printer name from the URI...
   */
 
-  if (strchr(argv[0], ':') != NULL)
-    httpSeparate(argv[0], method, username, hostname, &port, resource);
-  else if (getenv("DEVICE_URI") != NULL)
+  if (getenv("DEVICE_URI") != NULL)
+    /* authentication information is only available in the env var */
     httpSeparate(getenv("DEVICE_URI"), method, username, hostname, &port,
                  resource);
+  else if (strchr(argv[0], ':') != NULL)
+    httpSeparate(argv[0], method, username, hostname, &port, resource);
   else
   {
     fputs("ERROR: Missing device URI on command-line and no DEVICE_URI environment variable!\n",
@@ -819,13 +820,13 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
     else if ((job_id_attr = ippFindAttribute(response, "job-id",
                                              IPP_TAG_INTEGER)) == NULL)
     {
-      fputs("INFO: Print file accepted - job ID unknown.\n", stderr);
+      fputs("NOTICE: Print file accepted - job ID unknown.\n", stderr);
       job_id = 0;
     }
     else
     {
       job_id = job_id_attr->values[0].integer;
-      fprintf(stderr, "INFO: Print file accepted - job ID %d.\n", job_id);
+      fprintf(stderr, "NOTICE: Print file accepted - job ID %d.\n", job_id);
     }
 
     if (response)
@@ -1378,5 +1379,5 @@ sigterm_handler(int sig)		/* I - Signal */
 
 
 /*
- * End of "$Id: ipp.c,v 1.87 2004/08/26 18:25:50 mike Exp $".
+ * End of "$Id: ipp.c,v 1.90 2005/01/03 19:29:44 mike Exp $".
  */
