@@ -1,9 +1,9 @@
 /*
- * "$Id: printers.c,v 1.30 2003/11/07 18:52:38 mike Exp $"
+ * "$Id: printers.c,v 1.33 2005/01/03 19:29:45 mike Exp $"
  *
  *   Printer status CGI for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2003 by Easy Software Products.
+ *   Copyright 1997-2005 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -15,9 +15,9 @@
  *       Attn: CUPS Licensing Information
  *       Easy Software Products
  *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3111 USA
+ *       Hollywood, Maryland 20636 USA
  *
- *       Voice: (301) 373-9603
+ *       Voice: (301) 373-9600
  *       EMail: cups-info@cups.org
  *         WWW: http://www.cups.org
  *
@@ -134,7 +134,7 @@ main(int  argc,				/* I - Number of command-line arguments */
 
         cgiSetVariable("DEFAULT_URI",
 	               ippRewriteURL(attr->values[0].string.text,
-		                     url, sizeof(url)));
+		                     url, sizeof(url), NULL));
       }
 
       ippDelete(response);
@@ -352,5 +352,5 @@ main(int  argc,				/* I - Number of command-line arguments */
 
 
 /*
- * End of "$Id: printers.c,v 1.30 2003/11/07 18:52:38 mike Exp $".
+ * End of "$Id: printers.c,v 1.33 2005/01/03 19:29:45 mike Exp $".
  */
