@@ -1,9 +1,9 @@
 /*
- * "$Id: template.c,v 1.30 2004/02/25 20:14:51 mike Exp $"
+ * "$Id: template.c,v 1.32 2005/01/03 19:29:45 mike Exp $"
  *
  *   CGI template function.
  *
- *   Copyright 1997-2004 by Easy Software Products.
+ *   Copyright 1997-2005 by Easy Software Products.
  *
  *   This program is free software; you can redistribute it and/or modify it
  *   under the terms of the GNU General Public License as published by the Free
@@ -383,7 +383,7 @@ cgi_copy(FILE *out,		/* I - Output file */
 	        strlcpy(s, innerval, sizeof(compare) - (s - compare));
             }
 	    else if ((innerval = cgiGetArray(innername, element)) == NULL)
-	      snprintf(s, sizeof(s), "{%s}", innername);
+	      snprintf(s, sizeof(compare) - (s - compare), "{%s}", innername);
 	    else
 	      strlcpy(s, innerval, sizeof(compare) - (s - compare));
 
@@ -488,5 +488,5 @@ cgi_puts(const char *s,
 
 
 /*
- * End of "$Id: template.c,v 1.30 2004/02/25 20:14:51 mike Exp $".
+ * End of "$Id: template.c,v 1.32 2005/01/03 19:29:45 mike Exp $".
  */
