@@ -1,9 +1,9 @@
 /*
- * "$Id: http.c,v 1.142 2004/08/18 15:36:34 mike Exp $"
+ * "$Id: http.c,v 1.144 2005/01/03 19:29:45 mike Exp $"
  *
  *   HTTP routines for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2004 by Easy Software Products, all rights reserved.
+ *   Copyright 1997-2005 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -15,7 +15,7 @@
  *       Attn: CUPS Licensing Information
  *       Easy Software Products
  *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3142 USA
+ *       Hollywood, Maryland 20636 USA
  *
  *       Voice: (301) 373-9600
  *       EMail: cups-info@cups.org
@@ -2156,17 +2156,25 @@ http_wait(http_t *http,			/* I - HTTP data */
       return (0);
   }
 
-  FD_SET(http->fd, http->input_set);
-
-  if (msec >= 0)
+  do
   {
-    timeout.tv_sec  = msec / 1000;
-    timeout.tv_usec = (msec % 1000) * 1000;
+    FD_SET(http->fd, http->input_set);
 
-    nfds = select(http->fd + 1, http->input_set, NULL, NULL, &timeout);
+    if (msec >= 0)
+    {
+      timeout.tv_sec  = msec / 1000;
+      timeout.tv_usec = (msec % 1000) * 1000;
+
+      nfds = select(http->fd + 1, http->input_set, NULL, NULL, &timeout);
+    }
+    else
+      nfds = select(http->fd + 1, http->input_set, NULL, NULL, NULL);
   }
-  else
-    nfds = select(http->fd + 1, http->input_set, NULL, NULL, NULL);
+#ifdef WIN32
+  while (nfds < 0 && WSAGetLastError() == WSAEINTR);
+#else
+  while (nfds < 0 && errno == EINTR);
+#endif /* WIN32 */
 
   FD_CLR(http->fd, http->input_set);
 
@@ -2549,5 +2557,5 @@ CDSAWriteFunc(SSLConnectionRef connection,	/* I  - SSL/TLS connection */
 
 
 /*
- * End of "$Id: http.c,v 1.142 2004/08/18 15:36:34 mike Exp $".
+ * End of "$Id: http.c,v 1.144 2005/01/03 19:29:45 mike Exp $".
  */
