@@ -1,9 +1,9 @@
 /*
- * "$Id: language.c,v 1.51 2004/08/19 13:54:38 mike Exp $"
+ * "$Id: language.c,v 1.53 2005/01/03 19:29:45 mike Exp $"
  *
  *   I18N/language support for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2004 by Easy Software Products.
+ *   Copyright 1997-2005 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -15,7 +15,7 @@
  *       Attn: CUPS Licensing Information
  *       Easy Software Products
  *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3142 USA
+ *       Hollywood, Maryland 20636 USA
  *
  *       Voice: (301) 373-9600
  *       EMail: cups-info@cups.org
@@ -267,7 +267,14 @@ cupsLangGet(const char *language)	/* I - Language or locale */
                   ptr ? ptr : "(null)"));
 
     if (!ptr || !strcmp(ptr, "C") || !strcmp(ptr, "POSIX"))
+#  ifdef LC_MESSAGES
+    {
+      ptr = setlocale(LC_MESSAGES, "");
+      setlocale(LC_CTYPE, "");
+    }
+#  else
       ptr = setlocale(LC_ALL, "");
+#  endif /* LC_MESSAGES */
 
     if (ptr)
     {
@@ -982,5 +989,5 @@ cups_cache_lookup(const char      *name,/* I - Name of locale */
 
 
 /*
- * End of "$Id: language.c,v 1.51 2004/08/19 13:54:38 mike Exp $".
+ * End of "$Id: language.c,v 1.53 2005/01/03 19:29:45 mike Exp $".
  */
