@@ -1,10 +1,10 @@
 /*
- * "$Id: usersys.c,v 1.24 2004/06/29 03:44:52 mike Exp $"
+ * "$Id: usersys.c,v 1.27 2005/01/03 19:29:45 mike Exp $"
  *
  *   User, system, and password routines for the Common UNIX Printing
  *   System (CUPS).
  *
- *   Copyright 1997-2004 by Easy Software Products.
+ *   Copyright 1997-2005 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -16,7 +16,7 @@
  *       Attn: CUPS Licensing Information
  *       Easy Software Products
  *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3142 USA
+ *       Hollywood, Maryland 20636 USA
  *
  *       Voice: (301) 373-9600
  *       EMail: cups-info@cups.org
@@ -123,7 +123,7 @@ cupsEncryption(void)
       if (fp != NULL)
       {
        /*
-	* Read the config file and look for a ServerName line...
+	* Read the config file and look for an Encryption line...
 	*/
 
 	while (cups_get_line(line, sizeof(line), fp) != NULL)
@@ -448,7 +448,7 @@ cups_get_line(char *buf,	/* I - Line buffer */
   if (bufptr < buf)
     return (NULL);
 
-  while (isspace(*bufptr & 255) && bufptr >= buf)
+  while (bufptr >= buf && isspace(*bufptr & 255))
     *bufptr-- = '\0';
 
   return (buf);
@@ -456,5 +456,5 @@ cups_get_line(char *buf,	/* I - Line buffer */
 
 
 /*
- * End of "$Id: usersys.c,v 1.24 2004/06/29 03:44:52 mike Exp $".
+ * End of "$Id: usersys.c,v 1.27 2005/01/03 19:29:45 mike Exp $".
  */
