@@ -380,8 +380,8 @@ void SampledFunction::transform(double *in, double *out) {
 
     // pull 2^m values out of the sample array
     for (j = 0; j < (1<<m); ++j) {
-      idx = e[j & 1][m - 1];
-      for (k = m - 2; k >= 0; --k) {
+      idx = 0;
+      for (k = m - 1; k >= 0; --k) {
 	idx = idx * sampleSize[k] + e[(j >> k) & 1][k];
       }
       idx = idx * n + i;
@@ -616,9 +616,13 @@ StitchingFunction::StitchingFunction(Object *funcObj, Dict *dict) {
 }
 
 StitchingFunction::StitchingFunction(StitchingFunction *func) {
+  int i;
+
   k = func->k;
   funcs = (Function **)gmalloc(k * sizeof(Function *));
-  memcpy(funcs, func->funcs, k * sizeof(Function *));
+  for (i = 0; i < k; ++i) {
+    funcs[i] = func->funcs[i]->copy();
+  }
   bounds = (double *)gmalloc((k + 1) * sizeof(double));
   memcpy(bounds, func->bounds, (k + 1) * sizeof(double));
   encode = (double *)gmalloc(2 * k * sizeof(double));
@@ -629,11 +633,13 @@ StitchingFunction::StitchingFunction(StitchingFunction *func) {
 StitchingFunction::~StitchingFunction() {
   int i;
 
+  if (funcs) {
   for (i = 0; i < k; ++i) {
     if (funcs[i]) {
       delete funcs[i];
     }
   }
+  }
   gfree(funcs);
   gfree(bounds);
   gfree(encode);
@@ -1241,7 +1247,7 @@ void PostScriptFunction::exec(PSStack *stack, int codePtr) {
 	} else {
 	  b2 = stack->popBool();
 	  b1 = stack->popBool();
-	  stack->pushReal(b1 && b2);
+	  stack->pushBool(b1 && b2);
 	}
 	break;
       case psOpAtan:
@@ -1308,8 +1314,8 @@ void PostScriptFunction::exec(PSStack *stack, int codePtr) {
 	stack->roll(2, 1);
 	break;
       case psOpExp:
-	r2 = stack->popInt();
-	r1 = stack->popInt();
+	r2 = stack->popNum();
+	r1 = stack->popNum();
 	stack->pushReal(pow(r1, r2));
 	break;
       case psOpFalse:
@@ -1421,7 +1427,7 @@ void PostScriptFunction::exec(PSStack *stack, int codePtr) {
 	if (stack->topIsInt()) {
 	  stack->pushInt(~stack->popInt());
 	} else {
-	  stack->pushReal(!stack->popBool());
+	  stack->pushBool(!stack->popBool());
 	}
 	break;
       case psOpOr:
@@ -1432,7 +1438,7 @@ void PostScriptFunction::exec(PSStack *stack, int codePtr) {
 	} else {
 	  b2 = stack->popBool();
 	  b1 = stack->popBool();
-	  stack->pushReal(b1 || b2);
+	  stack->pushBool(b1 || b2);
 	}
 	break;
       case psOpPop:
@@ -1450,7 +1456,7 @@ void PostScriptFunction::exec(PSStack *stack, int codePtr) {
 	}
 	break;
       case psOpSin:
-	stack->pushReal(cos(stack->popNum()));
+	stack->pushReal(sin(stack->popNum()));
 	break;
       case psOpSqrt:
 	stack->pushReal(sqrt(stack->popNum()));
@@ -1483,7 +1489,7 @@ void PostScriptFunction::exec(PSStack *stack, int codePtr) {
 	} else {
 	  b2 = stack->popBool();
 	  b1 = stack->popBool();
-	  stack->pushReal(b1 ^ b2);
+	  stack->pushBool(b1 ^ b2);
 	}
 	break;
       case psOpIf:
