@@ -30,9 +30,9 @@ public:
   ~GHash();
   void add(GString *key, void *val);
   void *lookup(GString *key);
-  void *lookup(char *key);
+  void *lookup(const char *key);
   void *remove(GString *key);
-  void *remove(char *key);
+  void *remove(const char *key);
   int getLength() { return len; }
   void startIter(GHashIter **iter);
   GBool getNext(GHashIter **iter, GString **key, void **val);
@@ -41,9 +41,9 @@ public:
 private:
 
   GHashBucket *find(GString *key, int *h);
-  GHashBucket *find(char *key, int *h);
+  GHashBucket *find(const char *key, int *h);
   int hash(GString *key);
-  int hash(char *key);
+  int hash(const char *key);
 
   GBool deleteKeys;		// set if key strings should be deleted
   int size;			// number of buckets
