@@ -1,9 +1,9 @@
 /*
- * "$Id: client.c,v 1.194 2004/09/12 18:49:26 mike Exp $"
+ * "$Id: client.c,v 1.197 2005/01/03 19:29:59 mike Exp $"
  *
  *   Client routines for the Common UNIX Printing System (CUPS) scheduler.
  *
- *   Copyright 1997-2004 by Easy Software Products, all rights reserved.
+ *   Copyright 1997-2005 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -15,7 +15,7 @@
  *       Attn: CUPS Licensing Information
  *       Easy Software Products
  *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3142 USA
+ *       Hollywood, Maryland 20636 USA
  *
  *       Voice: (301) 373-9600
  *       EMail: cups-info@cups.org
@@ -2909,9 +2909,13 @@ is_path_absolute(const char *path)	/* I - Input path */
   */
 
   while ((path = strstr(path, "/..")) != NULL)
+  {
     if (!path[3] || path[3] == '/')
       return (0);
 
+    path ++;
+  }
+
  /*
   * If we haven't found any relative paths, return 1 indicating an
   * absolute path...
@@ -3277,7 +3281,7 @@ pipe_command(client_t *con,		/* I - Client connection */
   * Create a pipe for the output...
   */
 
-  if (cupsdPipe(fds))
+  if (cupsdOpenPipe(fds))
   {
     ClearString(&query_string);
 
@@ -3389,8 +3393,7 @@ pipe_command(client_t *con,		/* I - Client connection */
     LogMessage(L_ERROR, "Unable to fork for CGI %s - %s", argv[0],
                strerror(errno));
 
-    close(fds[0]);
-    close(fds[1]);
+    cupsdClosePipe(fds);
     pid = 0;
   }
   else
@@ -3464,5 +3467,5 @@ CDSAWriteFunc(SSLConnectionRef connection,	/* I  - SSL/TLS connection */
 
 
 /*
- * End of "$Id: client.c,v 1.194 2004/09/12 18:49:26 mike Exp $".
+ * End of "$Id: client.c,v 1.197 2005/01/03 19:29:59 mike Exp $".
  */
