@@ -1,9 +1,9 @@
 /*
- * "$Id: cupsd.h,v 1.54 2004/09/09 15:10:18 mike Exp $"
+ * "$Id: cupsd.h,v 1.56 2005/01/03 19:29:59 mike Exp $"
  *
  *   Main header file for the Common UNIX Printing System (CUPS) scheduler.
  *
- *   Copyright 1997-2004 by Easy Software Products, all rights reserved.
+ *   Copyright 1997-2005 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -206,9 +206,10 @@ extern void	SetString(char **s, const char *v);
 extern void	SetStringf(char **s, const char *f, ...);
 extern void	StartServer(void);
 extern void	StopServer(void);
-extern int	cupsdPipe(int *fds);
+extern void	cupsdClosePipe(int *fds);
+extern int	cupsdOpenPipe(int *fds);
 
 
 /*
- * End of "$Id: cupsd.h,v 1.54 2004/09/09 15:10:18 mike Exp $".
+ * End of "$Id: cupsd.h,v 1.56 2005/01/03 19:29:59 mike Exp $".
  */
