@@ -1,5 +1,5 @@
 /*
- * "$Id: file.c,v 1.9 2004/09/09 15:10:18 mike Exp $"
+ * "$Id: file.c,v 1.12 2005/01/03 19:29:59 mike Exp $"
  *
  *   File functions for the Common UNIX Printing System (CUPS).
  *
@@ -8,7 +8,7 @@
  *   our own file functions allows us to provide transparent support of
  *   gzip'd print files, PPD files, etc.
  *
- *   Copyright 1997-2004 by Easy Software Products, all rights reserved.
+ *   Copyright 1997-2005 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -20,7 +20,7 @@
  *       Attn: CUPS Licensing Information
  *       Easy Software Products
  *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3142 USA
+ *       Hollywood, Maryland 20636 USA
  *
  *       Voice: (301) 373-9600
  *       EMail: cups-info@cups.org
@@ -458,9 +458,12 @@ cupsFileRead(cups_file_t *fp,		/* I - CUPS file */
   * Range check input...
   */
 
-  if (!fp || !buf || bytes <= 0 || fp->mode != 'r')
+  if (!fp || !buf || bytes < 0 || fp->mode != 'r')
     return (-1);
 
+  if (bytes == 0)
+    return (0);
+
  /*
   * Loop until all bytes are read...
   */
@@ -620,9 +623,12 @@ cupsFileWrite(cups_file_t *fp,		/* I - CUPS file */
   * Range check input...
   */
 
-  if (!fp || !buf || bytes <= 0 || fp->mode != 'w')
+  if (!fp || !buf || bytes < 0 || fp->mode != 'w')
     return (-1);
 
+  if (bytes == 0)
+    return (0);
+
  /*
   * Write the buffer...
   */
@@ -987,5 +993,5 @@ cups_write(int        fd,		/* I - File descriptor */
 
 
 /*
- * End of "$Id: file.c,v 1.9 2004/09/09 15:10:18 mike Exp $".
+ * End of "$Id: file.c,v 1.12 2005/01/03 19:29:59 mike Exp $".
  */
