@@ -1,9 +1,9 @@
 /*
- * "$Id: printers.c,v 1.162 2004/06/29 03:27:35 mike Exp $"
+ * "$Id: printers.c,v 1.167 2005/01/03 19:29:59 mike Exp $"
  *
  *   Printer routines for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2004 by Easy Software Products, all rights reserved.
+ *   Copyright 1997-2005 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -15,7 +15,7 @@
  *       Attn: CUPS Licensing Information
  *       Easy Software Products
  *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3142 USA
+ *       Hollywood, Maryland 20636 USA
  *
  *       Voice: (301) 373-9600
  *       EMail: cups-info@cups.org
@@ -43,6 +43,7 @@
  *   ValidateDest()         - Validate a printer/class destination.
  *   WritePrintcap()        - Write a pseudo-printcap file for older
  *                            applications that need it...
+ *   cupsdSanitizeURI()     - Sanitize a device URI...
  *   write_irix_config()    - Update the config files used by the IRIX
  *                            desktop tools.
  *   write_irix_state()     - Update the status files used by IRIX printing
@@ -561,13 +562,6 @@ DeletePrinter(printer_t *p,		/* I - Printer to delete */
     return;
   }
 
- /*
-  * If this printer is the next for browsing, point to the next one...
-  */
-
-  if (p == BrowseNext)
-    BrowseNext = p->next;
-
  /*
   * Remove the printer from the list...
   */
@@ -585,6 +579,13 @@ DeletePrinter(printer_t *p,		/* I - Printer to delete */
 
   StopPrinter(p, update);
 
+ /*
+  * If this printer is the next for browsing, point to the next one...
+  */
+
+  if (p == BrowseNext)
+    BrowseNext = p->next;
+
  /*
   * Remove the dummy interface/icon/option files under IRIX...
   */
@@ -625,7 +626,7 @@ DeletePrinter(printer_t *p,		/* I - Printer to delete */
   * Remove this printer from any classes...
   */
 
-  if (!(p->type & (CUPS_PRINTER_CLASS | CUPS_PRINTER_IMPLICIT)))
+  if (!(p->type & CUPS_PRINTER_IMPLICIT))
     DeletePrinterFromClasses(p);
 
  /*
@@ -1150,11 +1151,7 @@ void
 SetPrinterAttrs(printer_t *p)		/* I - Printer to setup */
 {
   char		uri[HTTP_MAX_URI];	/* URI for printer */
-  char		method[HTTP_MAX_URI],	/* Method portion of URI */
-		username[HTTP_MAX_URI],	/* Username portion of URI */
-		host[HTTP_MAX_URI],	/* Host portion of URI */
-		resource[HTTP_MAX_URI];	/* Resource portion of URI */
-  int		port;			/* Port portion of URI */
+  char		resource[HTTP_MAX_URI];	/* Resource portion of URI */
   int		i;			/* Looping var */
   char		filename[1024];		/* Name of PPD file */
   int		num_media;		/* Number of media options */
@@ -1360,12 +1357,7 @@ SetPrinterAttrs(printer_t *p)		/* I - Printer to setup */
         * http://..., ipp://..., etc.
 	*/
 
-        httpSeparate(p->device_uri, method, username, host, &port, resource);
-	if (port)
-	  snprintf(uri, sizeof(uri), "%s://%s:%d%s", method, host, port,
-	           resource);
-	else
-	  snprintf(uri, sizeof(uri), "%s://%s%s", method, host, resource);
+        cupsdSanitizeURI(p->device_uri, uri, sizeof(uri));
       }
       else
       {
@@ -2175,6 +2167,74 @@ WritePrintcap(void)
 }
 
 
+/*
+ * 'cupsdSanitizeURI()' - Sanitize a device URI...
+ */
+
+char *					/* O - New device URI */
+cupsdSanitizeURI(const char *uri,	/* I - Original device URI */
+                 char       *buffer,	/* O - New device URI */
+                 int        buflen)	/* I - Size of new device URI buffer */
+{
+  char	*start,				/* Start of data after scheme */
+	*slash,				/* First slash after scheme:// */
+	*ptr;				/* Pointer into user@host:port part */
+
+
+ /*
+  * Range check input...
+  */
+
+  if (!uri || !buffer || buflen < 2)
+    return (NULL);
+
+ /*
+  * Copy the device URI to the new buffer...
+  */
+
+  strlcpy(buffer, uri, buflen);
+
+ /*
+  * Find the end of the scheme:// part...
+  */
+
+  if ((ptr = strchr(buffer, ':')) == NULL)
+    return (buffer);			/* No scheme: part... */
+
+  for (start = ptr + 1; *start; start ++)
+    if (*start != '/')
+      break;
+
+ /*
+  * Find the next slash (/) in the URI...
+  */
+
+  if ((slash = strchr(start, '/')) == NULL)
+    slash = start + strlen(start);	/* No slash, point to the end */
+
+ /*
+  * Check for an @ sign before the slash...
+  */
+
+  if ((ptr = strchr(start, '@')) != NULL && ptr < slash)
+  {
+   /*
+    * Found an @ sign and it is before the resource part, so we have
+    * an authentication string.  Copy the remaining URI over the
+    * authentication string...
+    */
+
+    cups_strcpy(start, ptr + 1);
+  }
+
+ /*
+  * Return the new device URI...
+  */
+
+  return (buffer);
+}
+
+
 #ifdef __sgi
 /*
  * 'write_irix_config()' - Update the config files used by the IRIX
@@ -2431,5 +2491,5 @@ write_irix_state(printer_t *p)		/* I - Printer to update */
 
 
 /*
- * End of "$Id: printers.c,v 1.162 2004/06/29 03:27:35 mike Exp $".
+ * End of "$Id: printers.c,v 1.167 2005/01/03 19:29:59 mike Exp $".
  */
