@@ -1,9 +1,9 @@
 /*
- * "$Id: printers.h,v 1.39 2004/08/23 18:00:59 mike Exp $"
+ * "$Id: printers.h,v 1.41 2005/01/03 19:29:59 mike Exp $"
  *
  *   Printer definitions for the Common UNIX Printing System (CUPS) scheduler.
  *
- *   Copyright 1997-2004 by Easy Software Products, all rights reserved.
+ *   Copyright 1997-2005 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -126,7 +126,10 @@ extern const char	*ValidateDest(const char *hostname,
 			              cups_ptype_t *dtype);
 extern void		WritePrintcap(void);
 
+extern char		*cupsdSanitizeURI(const char *uri, char *buffer,
+			                  int buflen);
+
 
 /*
- * End of "$Id: printers.h,v 1.39 2004/08/23 18:00:59 mike Exp $".
+ * End of "$Id: printers.h,v 1.41 2005/01/03 19:29:59 mike Exp $".
  */
