@@ -1,9 +1,9 @@
 /*
- * "$Id: ipptest.c,v 1.17 2004/02/25 20:14:54 mike Exp $"
+ * "$Id: ipptest.c,v 1.19 2005/01/03 19:30:00 mike Exp $"
  *
  *   IPP test command for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2004 by Easy Software Products.
+ *   Copyright 1997-2005 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -15,9 +15,9 @@
  *       Attn: CUPS Licensing Information
  *       Easy Software Products
  *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3111 USA
+ *       Hollywood, Maryland 20636 USA
  *
- *       Voice: (301) 373-9603
+ *       Voice: (301) 373-9600
  *       EMail: cups-info@cups.org
  *         WWW: http://www.cups.org
  *
@@ -83,7 +83,7 @@ main(int  argc,				/* I - Number of command-line arguments */
 
   if (argc < 3)
   {
-    fputs("Usage: testipp URL testfile [ ... testfileN ]\n", stderr);
+    fputs("Usage: ipptest URL testfile [ ... testfileN ]\n", stderr);
     return (1);
   }
 
@@ -822,5 +822,5 @@ print_attr(ipp_attribute_t *attr)	/* I - Attribute to print */
 
 
 /*
- * End of "$Id: ipptest.c,v 1.17 2004/02/25 20:14:54 mike Exp $".
+ * End of "$Id: ipptest.c,v 1.19 2005/01/03 19:30:00 mike Exp $".
  */
